#include <BootstrapList.h>
#include <UnderlayConfigurator.h>

#include <OverSwarm.h>
#include <BaseOverSwarmOverlay.h>

#include "OverSwarmTemplate.h"

namespace oversim {

Define_Module(OverSwarmTemplate);

/* Implementation goes there */
#include "OverSwarmTemplate_Implementation.h"

OverSwarmTemplate::OverSwarmTemplate()
{
	this->utils = NULL;
	this->internalTimer = NULL;
	this->isMaster = false;
}

void OverSwarmTemplate::initializeOverlay(int stage)
{
	if (stage != MIN_STAGE_OVERLAY) return;

	/* Setup submodules pointers */
	utils = check_and_cast<OvSwTemplateSampleLib*>
	                  (getParentModule()->getSubmodule("utils"));

	/* Periodic events */
	internalTimer = new cMessage("Event timer");
	scheduleAt(simTime() + 2, internalTimer);

	utils->initialize(stage);

	if (bootstrapList->getBootstrapNode().isUnspecified()) {
		this->isMaster = true;
	}

	/* We are ready */
	 setOverlayReady(true);
}

OverSwarmTemplate::~OverSwarmTemplate()
{
	cancelAndDelete(internalTimer);
}

void OverSwarmTemplate::handleTimerEvent(cMessage* msg)
{
	if (msg == internalTimer) {
			/* Reschedule */
			scheduleAt(simTime() + 2, internalTimer);

			/* Wait until the underlay is ready */
			if (underlayConfigurator->isInInitPhase()) return;

			/* Do what is needed */
			if (this->isMaster) {
				executeSayHello(new OvSwString("Joe"), NULL);
			}
	}
}

}
